/**********************************************************************
* FILENAME
*   blocks.h
*
* DESCRIPTION
*   Definition of block processing constants for EDMA-based working 
*   buffer environment
*
*  Rev 1.10  Oct. 27th, 2001
*  Rev 1.11  Nov. 09th, 2001    
*  Rev 2.00  Aug. 25th, 2005    C.Feldbauer <feldbauer@tugraz.at>
*
*----------------------------------------------------------------------
*  HISTORY
*  Rev 1.00  Created by C.Feldbauer
*  Rev 1.10  Added constants for modularity
*  Rev 1.11  Added workbuf alignment
*  Rev 2.00  New meaning of OVERLAP; OLA and NCHAN added
**********************************************************************/

#ifndef __BLOCKS__
#define __BLOCKS__

// these 2 set the size of the working buffer...
#define HOPSZ 128               // DMA block size -> interrupt interval
#define OVERLAP 0              // Overlap factor: actual overlap factor = OVERLAP/(OVERLAP+1)
                               // OVERLAP=0 -> no overlap (standard block processing)
                               // OVERLAP=1 -> 50% overlap of consecutive working buffers
                               // OVERLAP=3 -> 75% overlap of consecutive working buffers
                               // ...
#define OLA 0                  // flag to activate summation of overlapping outgoing buffers
                               // (has no meaning unless OVERLAP>0)
                               // OLA=0 -> no summation (outgoing signal is a concatenation 
                               // of the first HOPSZ samples of consecutive working buffers)
                               // e.g., for common, direct FIR implementations or OLS fast 
                               // convolution block filters
                               // OLA=1 -> summation of overlapping outgoing blocks
                               // e.g., for OLA block filters or lapped transforms such as
                               // STFT, MLT or MDCT (i.e., oversampled transform filterbanks)


#define NWBLOCKS (OVERLAP+1)   // number of blocks (each of size HOPSZ) for working buffer
#define NBLOCKS (NWBLOCKS+1)   // number of blocks for I/O DMA circular buffers
                               // OVERLAP=0 -> NBLOCKS=2 -> standard 'double buffering'
                               // (or 'ping-pong buffering')
                               // There is always one extra block to enable the DMA controller 
                               // to work in the background.


// constants you should know and use...
#define BLKSZ (NWBLOCKS*HOPSZ) // size of the working buffer in samples
#define FFTSZ BLKSZ            // the same as BLKSZ




// don't modify...
#define NCHAN 2                // codec provides stereo data
#define DATSZ 2                // codec and EDMA is set up for 16bit data
#define WBALIGN 4              // alignment in bytes of the working buffer (for optimization 
                               // or restriction issues)



// other useful constants...
#define Q15 32768
#define Q14 Q15/2
#define Q13 Q14/2
#define Q12 Q13/2
#define Q11 Q12/2
#define Q10 Q11/2




// PROTOTYPES
// the following two functions must be defined in the user's files

void init( void);                   // e.g. for object's initialization
                                    // called in main()
void processing( void * buffer);    // contains the actual signal processing
                                    // called in the ISR for EDMA interrupts


#endif
